@@ -26,13 +26,11 @@ NOT to be confused with the Docker builder...
 """
 
 import re
-from collections import defaultdict, deque
+from collections import defaultdict
 import os
-from os.path import (
-    abspath, basename, dirname, isfile, join, relpath, samefile, split)
+from os.path import abspath, basename, dirname, isfile, join
 from pathlib import PurePath
 import platform
-import shutil
 
 from ostrich.utils.path import commonpath
 from ostrich.utils.proc import run, PIPE, CalledProcessError
@@ -47,16 +45,16 @@ from .builders.ruby import format_gem_specifier
 from .pkgmgmt import (
     format_apt_specifier, format_pypi_specifier, parse_apt_repository)
 from .target_utils import ImageCachingBehavior
-from .utils import link_node, rmtree, yprint
+from .utils import link_node, rmtree
 
 
 logger = make_logger(__name__)
 
 
 KNOWN_RUNTIME_PARAMS = frozenset((
-        'ports', 'volumes', 'container_name', 'daemonize', 'interactive',
-        'term', 'auto_it', 'rm', 'env', 'work_dir', 'impersonate', 'network',
-        'runtime'))
+        'ports', 'volumes', 'container_name', 'daemonize', 'devices',
+        'interactive', 'term', 'auto_it', 'rm', 'env', 'work_dir', 'gpus',
+        'impersonate', 'network', 'runtime'))
 
 
 def make_pip_requirements(pip_requirements: set, pip_req_file_path: str):
@@ -170,19 +168,27 @@ def handle_build_cache(
     and non-trivial operations that are not usually expected from functions
     with such names.
     """
-    if icb.pull_if_cached or (icb.pull_if_not_cached and
-                              get_cached_image_id(icb.remote_image) is None):
+    # janky optimization ahead! (because `get_cached_image_id` is expensive)
+    # a bunch of ugly conditions that result the evaluation of
+    # `get_cached_image_id`
+    image_id = None
+    if (
+            ((not icb.pull_if_cached) and icb.pull_if_not_cached) or
+            icb.skip_build_if_cached or
+            (not icb.allow_build_if_not_cached)
+       ):
+        image_id = get_cached_image_id(icb.remote_image)
+    if icb.pull_if_cached or (icb.pull_if_not_cached and image_id is None):
         try:
             pull_docker_image(icb.remote_image, conf.docker_pull_cmd)
+            image_id = get_cached_image_id(icb.remote_image)
         except CalledProcessError:
             pass
     local_image = '{}:{}'.format(name, tag)
-    if (icb.skip_build_if_cached and
-            get_cached_image_id(icb.remote_image) is not None):
+    if icb.skip_build_if_cached and image_id is not None:
         tag_docker_image(icb.remote_image, local_image)
-        return get_cached_image_id(local_image)
-    if ((not icb.allow_build_if_not_cached) and
-            get_cached_image_id(icb.remote_image) is None):
+        return image_id
+    if (not icb.allow_build_if_not_cached) and image_id is None:
         raise RuntimeError('No cached image for {}'.format(local_image))
     return None
 
@@ -211,23 +217,31 @@ def format_docker_run_params(params: dict):
         param_strings.extend(['-p', port])
     for volume in params['volumes']:
         param_strings.extend(['-v', volume])
+    for device in params['devices']:
+        param_strings.extend(['--device', device])
     if params.get('work_dir'):
         param_strings.extend(['-w', params['work_dir']])
     for var, value in params['env'].items():
-        param_strings.extend(['-e', '{}="{}"'.format(var, value)])
+        param_strings.extend(['-e', '{}={}'.format(var, value)])
     if params.get('network'):
         param_strings.extend(['--net', params['network']])
     if params.get('runtime'):
         param_strings.extend(['--runtime', params['runtime']])
+    if params.get('gpus'):
+        param_strings.extend(['--gpus', params['gpus']])
 
     return param_strings
 
 
+def format_cmd_args(cmd_args):
+    return ('"{}"'.format(arg) for arg in cmd_args)
+
+
 def _replace_env_var(path):
     """
     replace all $VAR with the environment variable value
     """
-    return re.sub('\$[a-zA-Z_][a-zA-Z0-9_]*',
+    return re.sub(r'\$[a-zA-Z_][a-zA-Z0-9_]*',
                   lambda match: os.environ[match.group()[1:]], path)
 
 
@@ -244,10 +258,14 @@ def update_runtime_params(runtime_params: dict, new_rt_param: dict,
     if replace_env:
         runtime_params['volumes'].extend(
             map(_replace_env_var, listify(new_rt_param.get('volumes'))))
+        runtime_params['devices'].extend(
+            map(_replace_env_var, listify(new_rt_param.get('devices'))))
     else:
         runtime_params['volumes'].extend(listify(
             new_rt_param.get('volumes')))
-    runtime_params['env'].update(dict(runtime_params.get('env', {})))
+        runtime_params['devices'].extend(listify(
+            new_rt_param.get('devices')))
+    runtime_params['env'].update(dict(new_rt_param.get('env', {})))
     for param in ('container_name', 'daemonize', 'interactive', 'term',
                   'auto_it', 'rm', 'work_dir', 'impersonate', 'network'):
         if param in new_rt_param:
@@ -266,8 +284,11 @@ def extend_runtime_params(runtime_params, deps, extra_params=None,
     if replace_env:
         runtime_params['volumes'] = list(map(
             _replace_env_var, listify(runtime_params.get('volumes'))))
+        runtime_params['devices'] = list(map(
+            _replace_env_var, listify(runtime_params.get('devices'))))
     else:
         runtime_params['volumes'] = listify(runtime_params.get('volumes'))
+        runtime_params['devices'] = listify(runtime_params.get('devices'))
     runtime_params['env'] = dict(runtime_params.get('env', {}))
 
     for dep in deps:
@@ -518,13 +539,18 @@ def build_docker_image(
             os.makedirs(workspace_packages_dir)
             run_installers = []
             for custom_installer_desc in packages:
-                target_name, install_script, package = custom_installer_desc
+                target_name, script, args, package = custom_installer_desc
                 package_tar = basename(package)
                 link_node(package, join(workspace_packages_dir, package_tar))
+                exec_cmd = 'bash -s'
+                if args:
+                    exec_cmd = '{} -- {}'.format(
+                        exec_cmd,
+                        ' '.join(format_cmd_args(args)))
                 run_installers.extend([
                     'tar -xf {0}/{1} -C {0}'.format(tmp_install, package_tar),
                     'cd {}/{}'.format(tmp_install, target_name),
-                    'cat {} | tr -d \'\\r\' | bash'.format(install_script),
+                    'cat {} | tr -d \'\\r\' | {}'.format(script, exec_cmd),
                 ])
             dockerfile.extend([
                 'COPY {} {}\n'.format(packages_dir, tmp_install),
@@ -580,9 +606,6 @@ def build_docker_image(
                 ' '.join('"{}"="{}"'.format(key, value)
                          for key, value in sorted(effective_labels.items()))))
 
-    def format_docker_cmd(docker_cmd):
-        return ('"{}"'.format(cmd) for cmd in docker_cmd)
-
     if run_user:
         dockerfile.append('USER {}\n'.format(run_user))
 
@@ -598,7 +621,7 @@ def build_docker_image(
             entrypoint = ['tini', '--'] + entrypoint
         dockerfile.append(
             'ENTRYPOINT [{}]\n'.format(
-                ', '.join(format_docker_cmd(entrypoint))))
+                ', '.join(format_cmd_args(entrypoint))))
 
     # Add CMD (one layer)
     if cmd:
@@ -606,7 +629,7 @@ def build_docker_image(
         if full_path_cmd:
             cmd[0] = (PurePath('/usr/src/app') / cmd[0]).as_posix()
         dockerfile.append(
-            'CMD [{}]\n'.format(', '.join(format_docker_cmd(cmd))))
+            'CMD [{}]\n'.format(', '.join(format_cmd_args(cmd))))
 
     # TODO(itamar): write only if changed?
     with open(dockerfile_path, 'w') as dockerfile_f:
@@ -646,8 +669,8 @@ def build_docker_image(
 
         runtime_params = extend_runtime_params(
             runtime_params, deps, build_context.conf.runtime_params)
-        with open(join(dirname(abspath(__file__)),
-                  'ybtbin.sh.tmpl'), 'r') as tmpl_f:
+        with open(join(dirname(abspath(__file__)), 'ybtbin.sh.tmpl'),
+                  'r') as tmpl_f:
             ybt_bin = tmpl_f.read().format(
                 image_name=docker_image, image_id=image_id,
                 docker_opts=' '.join(format_docker_run_params(runtime_params)),
